@@ -15,11 +15,10 @@
  *
  **********************************************************************
  *
- * Last port: operation/buffer/BufferBuilder.java r378 (JTS-1.12)
+ * Last port: operation/buffer/BufferBuilder.java 149b38907 (JTS-1.12)
  *
  **********************************************************************/
 
-#include <geos/geom/CoordinateSequenceFactory.h>
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/Location.h>
 #include <geos/geom/Geometry.h>
@@ -29,13 +28,15 @@
 #include <geos/geom/MultiLineString.h>
 #include <geos/operation/buffer/BufferBuilder.h>
 #include <geos/operation/buffer/OffsetCurveBuilder.h>
-#include <geos/operation/buffer/OffsetCurveSetBuilder.h>
+#include <geos/operation/buffer/BufferCurveSetBuilder.h>
 #include <geos/operation/buffer/BufferSubgraph.h>
 #include <geos/operation/buffer/SubgraphDepthLocater.h>
-#include <geos/operation/overlay/OverlayOp.h>
+#include <geos/operation/overlayng/OverlayNG.h>
 #include <geos/operation/overlay/snap/SnapOverlayOp.h>
 #include <geos/operation/overlay/PolygonBuilder.h>
 #include <geos/operation/overlay/OverlayNodeFactory.h>
+#include <geos/operation/polygonize/Polygonizer.h>
+#include <geos/operation/union/UnaryUnionOp.h>
 #include <geos/operation/valid/RepeatedPointRemover.h>
 #include <geos/operation/linemerge/LineMerger.h>
 #include <geos/algorithm/LineIntersector.h>
@@ -71,7 +72,7 @@
 #define JTS_DEBUG 0
 #endif
 
-//using namespace std;
+//
 using namespace geos::geom;
 using namespace geos::geomgraph;
 using namespace geos::noding;
@@ -86,14 +87,14 @@ template <class Iterator>
 std::unique_ptr<Geometry>
 convertSegStrings(const GeometryFactory* fact, Iterator it, Iterator et)
 {
-    std::vector<Geometry*> lines;
+    std::vector<std::unique_ptr<Geometry>> lines;
     while(it != et) {
         const SegmentString* ss = *it;
-        LineString* line = fact->createLineString(ss->getCoordinates()->clone().release());
-        lines.push_back(line);
+        auto line = fact->createLineString(ss->getCoordinates()->clone());
+        lines.push_back(std::move(line));
         ++it;
     }
-    return std::unique_ptr<Geometry>(fact->buildGeometry(lines.begin(), lines.end()));
+    return fact->buildGeometry(lines.begin(), lines.end());
 }
 
 }
@@ -127,7 +128,7 @@ BufferBuilder::~BufferBuilder()
 }
 
 /*public*/
-Geometry*
+std::unique_ptr<Geometry>
 BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
                                      bool leftSide)
 {
@@ -140,7 +141,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
 
     // Nothing to do for a distance of zero
     if(distance == 0) {
-        return g->clone().release();
+        return g->clone();
     }
 
     // Get geometry factory and precision model.
@@ -158,16 +159,14 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
     BufferParameters modParams = bufParams;
     modParams.setEndCapStyle(BufferParameters::CAP_FLAT);
     modParams.setSingleSided(false); // ignore parameter for areal-only geometries
-    std::unique_ptr<Geometry> buf;
+
 
     // This is a (temp?) hack to workaround the fact that
-    // BufferBuilder BufferParamaters are immutable after
+    // BufferBuilder BufferParameters are immutable after
     // construction, while we want to force the end cap
     // style to FLAT for single-sided buffering
-    {
-        BufferBuilder tmp(modParams);
-        buf.reset(tmp.buffer(l, distance));
-    }
+    BufferBuilder tmpBB(modParams);
+    std::unique_ptr<Geometry> buf(tmpBB.buffer(l, distance));
 
     // Create MultiLineStrings from this polygon.
     std::unique_ptr<Geometry> bufLineString(buf->getBoundary());
@@ -194,7 +193,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
         CoordinateSequence* seq = lineList[i];
 
         // SegmentString takes ownership of CoordinateSequence
-        SegmentString* ss = new NodedSegmentString(seq, nullptr);
+        SegmentString* ss = new NodedSegmentString(seq, seq->hasZ(), seq->hasM(), nullptr);
         curveList.push_back(ss);
     }
     lineList.clear();
@@ -207,29 +206,25 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
     SegmentString::NonConstVect* nodedEdges = noder->getNodedSubstrings();
 
     // Create a geometry out of the noded substrings.
-    std::vector< Geometry* >* singleSidedNodedEdges =
-        new std::vector< Geometry* >();
-    singleSidedNodedEdges->reserve(nodedEdges->size());
+    std::vector<std::unique_ptr<Geometry>> singleSidedNodedEdges;
+    singleSidedNodedEdges.reserve(nodedEdges->size());
     for(std::size_t i = 0, n = nodedEdges->size(); i < n; ++i) {
         SegmentString* ss = (*nodedEdges)[i];
 
-        Geometry* tmp = geomFact->createLineString(
-                            ss->getCoordinates()->clone().release()
-                        );
+        auto tmp = geomFact->createLineString(ss->getCoordinates()->clone());
         delete ss;
 
-        singleSidedNodedEdges->push_back(tmp);
+        singleSidedNodedEdges.push_back(std::move(tmp));
     }
 
     delete nodedEdges;
 
-    for(size_t i = 0, n = curveList.size(); i < n; ++i) {
+    for(std::size_t i = 0, n = curveList.size(); i < n; ++i) {
         delete curveList[i];
     }
     curveList.clear();
 
-    std::unique_ptr<Geometry> singleSided(geomFact->createMultiLineString(
-            singleSidedNodedEdges));
+    auto singleSided = geomFact->createMultiLineString(std::move(singleSidedNodedEdges));
 
 #ifdef GEOS_DEBUG_SSB
     std::cerr << "edges|" << *singleSided << std::endl;
@@ -243,7 +238,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
     //       intersections with caps and joins curves
     using geos::operation::overlay::snap::SnapOverlayOp;
     std::unique_ptr<Geometry> intersectedLines = SnapOverlayOp::overlayOp(*singleSided, *bufLineString,
-            OverlayOp::opINTERSECTION);
+            overlayng::OverlayNG::INTERSECTION);
 
 #ifdef GEOS_DEBUG_SSB
     std::cerr << "intersection" << "|" << *intersectedLines << std::endl;
@@ -254,21 +249,21 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
     lineMerge.add(intersectedLines.get());
     auto mergedLines = lineMerge.getMergedLineStrings();
 
-    // Convert the result into a std::vector< Geometry* >.
-    std::vector< Geometry* >* mergedLinesGeom = new std::vector< Geometry* >();
+    // Convert the result into a vector of geometries
+    std::vector<std::unique_ptr<Geometry>> mergedLinesGeom;
     const Coordinate& startPoint = l->getCoordinatesRO()->front();
     const Coordinate& endPoint = l->getCoordinatesRO()->back();
     while(!mergedLines.empty()) {
         // Remove end points if they are a part of the original line to be
         // buffered.
         CoordinateSequence::Ptr coords(mergedLines.back()->getCoordinates());
-        if(nullptr != coords.get()) {
+        if(nullptr != coords) {
             // Use 98% of the buffer width as the point-distance requirement - this
             // is to ensure that the point that is "distance" +/- epsilon is not
             // included.
             //
             // Let's try and estimate a more accurate bound instead of just assuming
-            // 98%. With 98%, the episilon grows as the buffer distance grows,
+            // 98%. With 98%, the epsilon grows as the buffer distance grows,
             // so that at large distance, artifacts may skip through this filter
             // Let the length of the line play a factor in the distance, which is still
             // going to be bounded by 98%. Take 10% of the length of the line  from the buffer distance
@@ -279,9 +274,9 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
             // epsilon is removed.
             const double segLengthAllowance = 1.02 * distance;
 
-            size_t front = 0;
-            size_t back = coords->size() - 1;
-            size_t sz = back - front + 1;
+            std::size_t front = 0;
+            std::size_t back = coords->size() - 1;
+            std::size_t sz = back - front + 1;
 
             // Clean up the front of the list.
             // Loop until the line's end is not inside the buffer width from
@@ -292,7 +287,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
 
                 // Stop looping if there are no more points, or if the segment
                 // length is larger than the buffer width.
-                if(sz <= 1 || segLength > segLengthAllowance) {
+                if(segLength > segLengthAllowance) {
                     break;
                 }
 
@@ -303,7 +298,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
             }
             while(sz > 1 && coords->getAt(front).distance(endPoint) < ptDistAllowance) {
                 double segLength = coords->getAt(front).distance(coords->getAt(front + 1));
-                if(sz <= 1 || segLength > segLengthAllowance) {
+                if(segLength > segLengthAllowance) {
                     break;
                 }
                 front++;
@@ -313,7 +308,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
             while(sz > 1 && coords->getAt(back).distance(startPoint) < ptDistAllowance) {
                 double segLength = coords->getAt(back).distance(coords->getAt(back - 1));
 
-                if(sz <= 1 || segLength > segLengthAllowance) {
+                if(segLength > segLengthAllowance) {
                     break;
                 }
                 back--;
@@ -322,7 +317,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
             while(sz > 1 && coords->getAt(back).distance(endPoint) < ptDistAllowance) {
                 double segLength = coords->getAt(back).distance(coords->getAt(back - 1));
 
-                if(sz <= 1 || segLength > segLengthAllowance) {
+                if(segLength > segLengthAllowance) {
                     break;
                 }
                 back--;
@@ -332,11 +327,9 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
             if (sz > 1) {
                 if (sz < coords->size()) {
                     // Points were removed; make a new CoordinateSequence
-                    auto seqFactory = geomFact->getCoordinateSequenceFactory();
+                    auto newSeq = detail::make_unique<CoordinateSequence>(sz, coords->getDimension());
 
-                    auto newSeq = seqFactory->create(sz, coords->getDimension());
-
-                    for (size_t i = 0; i < sz; i++) {
+                    for (std::size_t i = 0; i < sz; i++) {
                         newSeq->setAt(coords->getAt(i + front), i);
                     }
 
@@ -344,7 +337,7 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
                 }
 
                 // Add the coordinates to the resultant line string.
-                mergedLinesGeom->push_back(geomFact->createLineString(coords.release()));
+                mergedLinesGeom.push_back(geomFact->createLineString(std::move(coords)));
             }
         }
 
@@ -359,26 +352,44 @@ BufferBuilder::bufferLineSingleSided(const Geometry* g, double distance,
     singleSided.reset();
     intersectedLines.reset();
 
-    if(mergedLinesGeom->size() > 1) {
-        return geomFact->createMultiLineString(mergedLinesGeom);
+    if(mergedLinesGeom.size() > 1) {
+        return geomFact->createMultiLineString(std::move(mergedLinesGeom));
     }
-    else if(mergedLinesGeom->size() == 1) {
-
-        Geometry* single = (*mergedLinesGeom)[0];
-        delete mergedLinesGeom;
+    else if(mergedLinesGeom.size() == 1) {
+        std::unique_ptr<Geometry> single = std::move(mergedLinesGeom[0]);
         return single;
     }
     else {
-        delete mergedLinesGeom;
-        return geomFact->createLineString().release();
+        return geomFact->createLineString();
     }
 }
 
 /*public*/
-Geometry*
+std::unique_ptr<Geometry>
 BufferBuilder::buffer(const Geometry* g, double distance)
 // throw(GEOSException *)
 {
+    // Single-sided buffer only works on single geometries
+    // so we'll need to do it individually and then union
+    // the result
+    if ( bufParams.isSingleSided() && g->getNumGeometries() > 1 )
+    {
+        std::vector< std::unique_ptr<Geometry> > geoms_to_delete;
+        for ( size_t i=0, n=g->getNumGeometries(); i<n; ++i )
+        {
+            // BufferBuilder class cannot be re-used, so
+            // we create a new one for each subgeom
+            BufferBuilder subbuilder(bufParams);
+            const Geometry *subgeom = g->getGeometryN(i);
+            std::unique_ptr<Geometry> subbuf = subbuilder.buffer(subgeom, distance);
+            geoms_to_delete.push_back( std::move(subbuf) );
+        }
+        std::vector< Geometry* > geoms;
+        for ( size_t i=0, n=geoms_to_delete.size(); i<n; ++i )
+            geoms.push_back( geoms_to_delete[i].get() );
+        return operation::geounion::UnaryUnionOp::Union(geoms);
+    }
+
     const PrecisionModel* precisionModel = workingPrecisionModel;
     if(precisionModel == nullptr) {
         precisionModel = g->getPrecisionModel();
@@ -392,17 +403,16 @@ BufferBuilder::buffer(const Geometry* g, double distance)
 
     {
         // This scope is here to force release of resources owned by
-        // OffsetCurveSetBuilder when we're doing with it
-
-        OffsetCurveBuilder curveBuilder(precisionModel, bufParams);
-        OffsetCurveSetBuilder curveSetBuilder(*g, distance, curveBuilder);
+        // BufferCurveSetBuilder when we're doing with it
+        BufferCurveSetBuilder curveSetBuilder(*g, distance, precisionModel, bufParams);
+        curveSetBuilder.setInvertOrientation(isInvertOrientation);
 
         GEOS_CHECK_FOR_INTERRUPTS();
 
         std::vector<SegmentString*>& bufferSegStrList = curveSetBuilder.getCurves();
 
 #if GEOS_DEBUG
-        std::cerr << "OffsetCurveSetBuilder got " << bufferSegStrList.size()
+        std::cerr << "BufferCurveSetBuilder got " << bufferSegStrList.size()
                   << " curves" << std::endl;
 #endif
         // short-circuit test
@@ -424,8 +434,8 @@ BufferBuilder::buffer(const Geometry* g, double distance)
     std::cerr << std::endl << edgeList << std::endl;
 #endif
 
-    Geometry* resultGeom = nullptr;
-    std::unique_ptr< std::vector<Geometry*> > resultPolyList;
+    std::unique_ptr<Geometry> resultGeom(nullptr);
+    std::vector<std::unique_ptr<Geometry>> resultPolyList;
     std::vector<BufferSubgraph*> subgraphList;
 
     try {
@@ -439,7 +449,7 @@ BufferBuilder::buffer(const Geometry* g, double distance)
 #if GEOS_DEBUG
         std::cerr << "Created " << subgraphList.size() << " subgraphs" << std::endl;
 #if GEOS_DEBUG > 1
-        for(size_t i = 0, n = subgraphList.size(); i < n; i++) {
+        for(std::size_t i = 0, n = subgraphList.size(); i < n; i++) {
             std::cerr << std::setprecision(10) << *(subgraphList[i]) << std::endl;
         }
 #endif
@@ -448,42 +458,42 @@ BufferBuilder::buffer(const Geometry* g, double distance)
         GEOS_CHECK_FOR_INTERRUPTS();
 
         {
-            // scope for earlier PolygonBuilder cleanupt
+            // scope for earlier PolygonBuilder cleanup
             PolygonBuilder polyBuilder(geomFact);
             buildSubgraphs(subgraphList, polyBuilder);
 
-            resultPolyList.reset(polyBuilder.getPolygons());
+            resultPolyList = polyBuilder.getPolygons();
         }
 
         // Get rid of the subgraphs, shouldn't be needed anymore
-        for(size_t i = 0, n = subgraphList.size(); i < n; i++) {
+        for(std::size_t i = 0, n = subgraphList.size(); i < n; i++) {
             delete subgraphList[i];
         }
         subgraphList.clear();
 
 #if GEOS_DEBUG
-        std::cerr << "PolygonBuilder got " << resultPolyList->size()
+        std::cerr << "PolygonBuilder got " << resultPolyList.size()
                   << " polygons" << std::endl;
 #if GEOS_DEBUG > 1
-        for(size_t i = 0, n = resultPolyList->size(); i < n; i++) {
-            std::cerr << (*resultPolyList)[i]->toString() << std::endl;
+        for(std::size_t i = 0, n = resultPolyList.size(); i < n; i++) {
+            std::cerr << resultPolyList[i]->toString() << std::endl;
         }
 #endif
 #endif
 
         // just in case ...
-        if(resultPolyList->empty()) {
+        if(resultPolyList.empty()) {
             return createEmptyResultGeometry();
         }
 
         // resultPolyList ownership transferred here
-        resultGeom = geomFact->buildGeometry(resultPolyList.release());
+        resultGeom = geomFact->buildGeometry(std::move(resultPolyList));
 
     }
     catch(const util::GEOSException& /* exc */) {
 
         // In case they're still around
-        for(size_t i = 0, n = subgraphList.size(); i < n; i++) {
+        for(std::size_t i = 0, n = subgraphList.size(); i < n; i++) {
             delete subgraphList[i];
         }
         subgraphList.clear();
@@ -491,6 +501,73 @@ BufferBuilder::buffer(const Geometry* g, double distance)
         throw;
     }
 
+    // Cleanup single-sided buffer artifacts, if needed
+    if ( bufParams.isSingleSided() )
+    {
+
+        // Get linework of input geom
+        const Geometry *inputLineString = g;
+        std::unique_ptr<Geometry> inputPolygonBoundary;
+        if ( g->getDimension() > 1 )
+        {
+            inputPolygonBoundary = g->getBoundary();
+            inputLineString = inputPolygonBoundary.get();
+        }
+
+#if GEOS_DEBUG
+        std::cerr << "Input linework: " << *inputLineString << std::endl;
+#endif
+
+        // Get linework of buffer geom
+        std::unique_ptr<Geometry> bufferBoundary = resultGeom->getBoundary();
+
+#if GEOS_DEBUG
+        std::cerr << "Buffer boundary: " << *bufferBoundary << std::endl;
+#endif
+
+        // Node all linework
+        using operation::overlayng::OverlayNG;
+        std::unique_ptr<Geometry> nodedLinework = OverlayNG::overlay(inputLineString, bufferBoundary.get(), OverlayNG::UNION);
+
+#if GEOS_DEBUG
+        std::cerr << "Noded linework: " << *nodedLinework << std::endl;
+#endif
+
+        using operation::polygonize::Polygonizer;
+        Polygonizer plgnzr;
+        plgnzr.add(nodedLinework.get());
+        std::vector<std::unique_ptr<geom::Polygon>> polys = plgnzr.getPolygons();
+
+#if GEOS_DEBUG
+        std::cerr << "Polygonization of noded linework returned: " << polys.size() << " polygons" << std::endl;
+#endif
+
+        if ( polys.size() > 1 )
+        {
+            // Only keep larger polygon
+            std::vector<std::unique_ptr<geom::Polygon>>::iterator
+                it,
+                itEnd = polys.end(),
+                biggestPolygonIterator = itEnd;
+            double maxArea = 0;
+            for ( it = polys.begin(); it != itEnd; ++it )
+            {
+                double area = (*it)->getArea();
+                if ( area > maxArea )
+                {
+                    biggestPolygonIterator = it;
+                    maxArea = area;
+                }
+            }
+
+            if ( biggestPolygonIterator != itEnd ) {
+                Geometry *gg = (*biggestPolygonIterator).release();
+                return std::unique_ptr<Geometry>( gg );
+            } // else there were no polygons formed...
+        }
+
+    }
+
     return resultGeom;
 }
 
@@ -518,7 +595,7 @@ BufferBuilder::getNoder(const PrecisionModel* pm)
 
 #if 0
     /* CoordinateArraySequence.cpp:84:
-     * virtual const geos::Coordinate& geos::CoordinateArraySequence::getAt(size_t) const:
+     * virtual const geos::Coordinate& geos::CoordinateArraySequence::getAt(std::size_t) const:
      * Assertion `pos<vect->size()' failed.
      */
 
@@ -543,7 +620,6 @@ BufferBuilder::computeNodedEdges(SegmentString::NonConstVect& bufferSegStrList,
 
 #if JTS_DEBUG
     geos::io::WKTWriter wktWriter;
-    wktWriter.setTrim(true);
     std::cerr << "before noding: "
               << wktWriter.write(
                   convertSegStrings(geomFact, bufferSegStrList.begin(),
@@ -651,7 +727,7 @@ BufferBuilder::createSubgraphs(PlanarGraph* graph, std::vector<BufferSubgraph*>&
 {
     std::vector<Node*> nodes;
     graph->getNodes(nodes);
-    for(size_t i = 0, n = nodes.size(); i < n; i++) {
+    for(std::size_t i = 0, n = nodes.size(); i < n; i++) {
         Node* node = nodes[i];
         if(!node->isVisited()) {
             BufferSubgraph* subgraph = new BufferSubgraph();
@@ -679,7 +755,7 @@ BufferBuilder::buildSubgraphs(const std::vector<BufferSubgraph*>& subgraphList,
     std::cerr << __FUNCTION__ << " got " << subgraphList.size() << " subgraphs" << std::endl;
 #endif
     std::vector<BufferSubgraph*> processedGraphs;
-    for(size_t i = 0, n = subgraphList.size(); i < n; i++) {
+    for(std::size_t i = 0, n = subgraphList.size(); i < n; i++) {
         BufferSubgraph* subgraph = subgraphList[i];
         Coordinate* p = subgraph->getRightmostCoordinate();
         assert(p);
@@ -715,11 +791,10 @@ BufferBuilder::buildSubgraphs(const std::vector<BufferSubgraph*>& subgraphList,
 }
 
 /*private*/
-geom::Geometry*
+std::unique_ptr<geom::Geometry>
 BufferBuilder::createEmptyResultGeometry() const
 {
-    geom::Geometry* emptyGeom = geomFact->createPolygon().release();
-    return emptyGeom;
+    return geomFact->createPolygon();
 }
 
 } // namespace geos.operation.buffer
