@@ -7,7 +7,7 @@
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU Lesser General Licence as published
- * by the Free Software Foundation. 
+ * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
  **********************************************************************
@@ -22,10 +22,9 @@ namespace geos {
 namespace triangulate { //geos.triangulate
 namespace quadedge { //geos.triangulate.quadedge
 
-LocateFailureException::LocateFailureException(std::string const&msg)
-{
-	util::GEOSException("LocateFailureException", msg);
-}
+LocateFailureException::LocateFailureException(std::string const& msg) :
+    util::GEOSException("LocateFailureException", msg)
+{}
 
 } //namespace geos.triangulate.quadedge
 } //namespace geos.triangulate
