@@ -55,5 +55,16 @@ void object::test<2>
     ensure(result_ == nullptr);
 }
 
+template<>
+template<>
+void object::test<3>()
+{
+    input_ = fromWKT("CIRCULARSTRING (0 0, 1 1, 2 0, 2 0, 2 0)");
+    ensure(input_);
+
+    result_ = GEOSRemoveRepeatedPoints(input_, 0);
+
+    ensure("curved geometry not supported", result_ == nullptr);
+}
 
 } // namespace tut
